/*
 * Suite of PhylogEnetiC Tools for Reticulate Evolution (SPECTRE)
 * Copyright (C) 2017  UEA School of Computing Sciences
 *
 * This program is free software: you can redistribute it and/or modify it under the term of the GNU General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package uk.ac.uea.cmp.spectre.core.ui.gui;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import java.io.File;
import java.net.URISyntaxException;

/**
 * Created by dan on 09/03/14.
 */
public class LookAndFeel {

    private static Logger log = LoggerFactory.getLogger(LookAndFeel.class);

    public static final String NIMBUS = "Nimbus";

    public static void setLookAndFeel(String lookAndFeel) {
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if (lookAndFeel.equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex) {
            log.error("Error setting look and feel for GUI", ex);
        }
    }

    public static String getLogoFilePath() throws URISyntaxException {

        File logo = new File(new File(LookAndFeel.class.getProtectionDomain().getCodeSource().getLocation().toURI().getPath()).getParentFile(), "etc/logo.png");
        if (!logo.exists()) {
            logo = FileUtils.toFile(LookAndFeel.class.getResource("/logo.png"));
        }
        return logo.getAbsolutePath();
    }
}
